@@ -1,4 +1,4 @@
-# $Id: twitter.py 1052 2014-10-14 20:11:46Z mwall $
+# $Id: twitter.py 1086 2014-10-22 18:07:09Z mwall $
 # Copyright 2014 Matthew Wall
 """
 Tweet weather data
@@ -11,31 +11,49 @@ twitter developer documentation to obtain these:
 Specifically, you will need four things:  app_key, app_key_secret, oauth_token,
 and oauth_token_secret.
 
-Installation:
-
-0) install twython
-   pip install twython
-
-1) put this file in the weewx user directory
-   cp twitter.py /home/weewx/bin/user
-
-2) add the following configuration stanza to weewx.conf
-
 [StdRESTful]
     [[Twitter]]
-        station = STATION_IDENTIFIER
         app_key = APP_KEY
         app_key_secret = APP_KEY_SECRET
         oauth_token = OAUTH_TOKEN
         oauth_token_secret = OAUTH_TOKEN_SECRET
 
-[Engines]
-    [[WxEngine]]
-        restful_services = ... , user.twitter.Twitter
+Tweets look something like this:
 
-3) restart weewx
-    sudo /etc/init.d/weewx stop
-    sudo /etc/init.d/weewx start
+STATION_IDENTIFIER: Ws: 0.0; Wd: -; Wg: 1.1; oT: 7.00; oH: 97.00; P: 1025.307; R: 0.000
+
+The STATION_IDENTIFIER is the first part of the station 'location' defined in
+weewx.conf.  To specify a different identifier for tweets, use the 'station'
+parameter.  For example:
+
+[StdRESTful]
+    [[Twitter]]
+        station = hal
+
+The 'format' parameter determines the tweet contents.  The default format is:
+
+format = {station:%.8s}: Ws: {windSpeed:%.1f}; Wd: {windDir:%03.0f}; Wg: {windGust:%.1f}; oT: {outTemp:%.1f}; oH: {outHumidity:%.2f}; P: {barometer:%.3f}; R: {rain:%.3f}
+
+To specify a different tweet message, use the format parameter.  For example,
+this would tweet only wind information:
+
+[StdRESTful]
+    [[Twitter]]
+        format = {station}: Ws: {windSpeed}; Wd: {windDir}; Wg: {windGust}
+
+If there is no value for an observation, the hyphen (-) will display.  If
+the observation does not exist, the observation label will not be replaced.
+If no format is specified for an observation, the default is used.
+For example:
+
+    Ws: {windSpeed}             ->  Ws: 12.3452343
+    Ws: {windSpeed:%.3f}        ->  Ws: 12.345
+
+The dateTime field is handled slightly differently.  For example:
+
+    ts: {dateTime}              ->  ts: 1413994070
+    ts: {dateTime:%X}           ->  ts: 16:07:50 22 Oct 2014
+    ts: {dateTime:%H:%M:%S}     ->  ts: 16:07:50
 
 By default, the units are those specified by the unit system in the
 StdConvert section of weewx.conf.  To specify a different unit system,
@@ -49,6 +67,7 @@ Possible values include US, METRIC, or METRICWX.
 """
 
 import Queue
+import re
 import sys
 import syslog
 import time
@@ -59,7 +78,7 @@ import weewx.units
 
 from twython import Twython, TwythonError, TwythonAuthError, TwythonRateLimitError
 
-VERSION = "0.4"
+VERSION = "0.5"
 
 def logmsg(level, msg):
     syslog.syslog(level, 'restx: Twitter: %s' % msg)
@@ -78,14 +97,15 @@ def _format(label, fmt, datum):
     return "%s: %s" % (label, s)
 
 class Twitter(weewx.restx.StdRESTbase):
+
+    _DEFAULT_FORMAT = '{station:%.8s}: Ws: {windSpeed:%.1f}; Wd: {windDir:%03.0f}; Wg: {windGust:%.1f}; oT: {outTemp:%.1f}; oH: {outHumidity:%.2f}; P: {barometer:%.3f}; R: {rain:%.3f}'
+    _DEFAULT_FORMAT_NONE = '-'
+
     def __init__(self, engine, config_dict):
-        """Initialize for posting to twitter.  This service supports the
-        standard restful options plus the following:
+        """This service recognizes standard restful options plus the following:
 
         Required parameters:
 
-        station: a short name to identify the weather station
-
         twitter authentication credentials:
         app_key
         app_key_secret
@@ -94,14 +114,25 @@ class Twitter(weewx.restx.StdRESTbase):
 
         Optional parameters:
 
+        station: a short name to identify the weather station
+        Default is the station location from [Station]
+
         unit_system: one of US, METRIC, or METRICWX
-        Default is None
+        Default is None; units will be those of the data in the database
+
+        format: indicates how the tweet should be rendered
+        Default contains basic weather data
+
+        format_None: indicates how a NULL value should be rendered
+        Default is -
+
+        binding: either loop or archive
+        Default is archive
         """
         super(Twitter, self).__init__(engine, config_dict)
         loginf('service version is %s' % VERSION)
         try:
             site_dict = weewx.restx.get_dict(config_dict, 'Twitter')
-            site_dict['station']
             site_dict['app_key']
             site_dict['app_key_secret']
             site_dict['oauth_token']
@@ -110,12 +141,20 @@ class Twitter(weewx.restx.StdRESTbase):
             logerr("Data will not be posted: Missing option %s" % e)
             return
 
-        # if a unit system was specified, get the weewx constant for it
+        # default the station name
+        site_dict.setdefault('station', engine.stn_info.location)
+
+        # if a unit system was specified, get the weewx constant for it.
+        # do it here so a bogus unit system will cause weewx to die
+        # immediately, not simply cause the twitter thread to crap out.
         usn = site_dict.get('unit_system', None)
         if usn is not None:
             site_dict['unit_system'] = weewx.units.unit_constants[usn]
             loginf('units will be converted to %s' % usn)
 
+        site_dict.setdefault('format', self._DEFAULT_FORMAT)
+        site_dict.setdefault('format_None', self._DEFAULT_FORMAT_NONE)
+
         # we can bind to archive or loop events, default to archive
         self.binding = site_dict.get('binding', 'archive')
 
@@ -136,21 +175,9 @@ class Twitter(weewx.restx.StdRESTbase):
         self.loop_queue.put(event.record)
 
 class TwitterThread(weewx.restx.RESTThread):
-    _FORMATS = {
-        "barometer":   ["P",  "%.3f"],
-        "inTemp":      ["iT", "%.2f"],
-        "outTemp":     ["oT", "%.2f"],
-        "inHumidity":  ["iH", "%.2f"],
-        "outHumidity": ["oH", "%.2f"],
-        "windSpeed":   ["Ws", "%.1f"],
-        "windGust":    ["Wg", "%.1f"],
-        "windDir":     ["Wd", "%.0f"],
-        "rain":        ["R",  "%.3f"],
-        }
-
     def __init__(self, queue, 
-                 station,
                  app_key, app_key_secret, oauth_token, oauth_token_secret,
+                 station, format, format_None,
                  unit_system=None, 
                  log_success=True, log_failure=True,
                  post_interval=None, max_backlog=sys.maxint, stale=None,
@@ -166,33 +193,61 @@ class TwitterThread(weewx.restx.RESTThread):
                                             max_tries=max_tries,
                                             timeout=timeout,
                                             retry_wait=retry_wait)
-        self.unit_system = unit_system
-        self.station = station
         self.app_key = app_key
         self.app_key_secret = app_key_secret
         self.oauth_token = oauth_token
         self.oauth_token_secret = oauth_token_secret
+        self.station = station
+        self.format = format
+        self.format_None = format_None
+        self.unit_system = unit_system
+
+    def format_tweet(self, record):
+        msg = self.format
+        for obs in record:
+            oldstr = None
+            fmt = '%s'
+            pattern = "{%s}" % obs
+            m = re.search(pattern, msg)
+            if m:
+                oldstr = m.group(0)
+            else:
+                pattern = "{%s:([^}]+)}" % obs
+                m = re.search(pattern, msg)
+                if m:
+                    oldstr = m.group(0)
+                    fmt = m.group(1)
+            if oldstr is not None:
+                if obs == 'dateTime':
+                    newstr = time.strftime(fmt, time.gmtime(record[obs]))
+                elif record[obs] is None:
+                    newstr = self.format_None
+                else:
+                    newstr = fmt % record[obs]
+                msg = msg.replace(oldstr, newstr)
+        return msg
 
     def process_record(self, record, dummy_archive):
         if self.unit_system is not None:
             record = weewx.units.to_std_system(record, self.unit_system)
-        s = []
-        s.append("%s: ts: %s" % (self.station, record['dateTime']))
-        for obs in self._FORMATS:
-            if obs in record:
-                s.append(_format(self._FORMATS[obs][0], self._FORMATS[obs][1],
-                                 record[obs]))
-        wstr = '; '.join(s)
+        record['station'] = self.station
+
+        msg = self.format_tweet(record)
+        logdbg(msg)
+
+        # now do the posting
         ntries = 0
         while ntries < self.max_tries:
             ntries += 1
             try:
                 twitter = Twython(self.app_key, self.app_key_secret,
                                   self.oauth_token, self.oauth_token_secret)
-                twitter.update_status(status=wstr)
+                twitter.update_status(status=msg)
                 return
-            except (TwythonError, TwythonAuthError, TwythonRateLimitError), e:
-                logerr("Failed attempt %d of %d to tweet: %s" %
+            except TwythonAuthError, e:
+                raise weewx.restx.FailedPost("Authorization failed: %s" % e)
+            except (TwythonError, TwythonRateLimitError), e:
+                logerr("Failed attempt %d of %d: %s" %
                        (ntries, self.max_tries, e))
                 logdbg("Waiting %d seconds before retry" % self.retry_wait)
                 time.sleep(self.retry_wait)
